uniform sampler2D texInput;
uniform sampler2D texBlend;
uniform highp float blendPower;

varying highp vec2 texCoord;
varying highp vec2 texCoord2;


highp vec4 Desaturate(highp vec3 color, highp float Desaturation)
{
    highp vec3 grayXfer = vec3(0.3, 0.59, 0.11);
    highp vec3 gray = vec3(dot(grayXfer, color));
    return vec4(mix(color, gray, Desaturation), 1.0);
}




highp vec3 RGBToHSL(highp vec3 color)
{
    highp vec3 hsl; // init to 0 to avoid warnings ? (and reverse if + remove first part)
    
    highp float fmin = min(min(color.r, color.g), color.b);    //Min. value of RGB
    highp float fmax = max(max(color.r, color.g), color.b);    //Max. value of RGB
    highp float delta = fmax - fmin;             //Delta RGB value
    
    hsl.z = (fmax + fmin) / 2.0; // Luminance
    
    if (delta == 0.0)      //This is a gray, no chroma...
    {
        hsl.x = 0.0;   // Hue
        hsl.y = 0.0;   // Saturation
    }
    else                                    //Chromatic data...
    {
        if (hsl.z < 0.5)
            hsl.y = delta / (fmax + fmin); // Saturation
        else
            hsl.y = delta / (2.0 - fmax - fmin); // Saturation
        
        highp float deltaR = (((fmax - color.r) / 6.0) + (delta / 2.0)) / delta;
        highp float deltaG = (((fmax - color.g) / 6.0) + (delta / 2.0)) / delta;
        highp float deltaB = (((fmax - color.b) / 6.0) + (delta / 2.0)) / delta;
        
        if (color.r == fmax )
            hsl.x = deltaB - deltaG; // Hue
        else if (color.g == fmax)
            hsl.x = (1.0 / 3.0) + deltaR - deltaB; // Hue
        else if (color.b == fmax)
            hsl.x = (2.0 / 3.0) + deltaG - deltaR; // Hue
        
        if (hsl.x < 0.0)
            hsl.x += 1.0; // Hue
        else if (hsl.x > 1.0)
            hsl.x -= 1.0; // Hue
    }
    
    return hsl;
}


highp float HueToRGB(highp float f1, highp float f2, highp float hue)
{
    if (hue < 0.0)
        hue += 1.0;
    else if (hue > 1.0)
        hue -= 1.0;
    highp float res;
    if ((6.0 * hue) < 1.0)
        res = f1 + (f2 - f1) * 6.0 * hue;
    else if ((2.0 * hue) < 1.0)
        res = f2;
    else if ((3.0 * hue) < 2.0)
        res = f1 + (f2 - f1) * ((2.0 / 3.0) - hue) * 6.0;
    else
        res = f1;
    return res;
}


highp vec3 HSLToRGB(highp vec3 hsl)
{
    highp vec3 rgb;
    
    if (hsl.y == 0.0)
        rgb = vec3(hsl.z); // Luminance
    else
    {
        highp float f2;
        
        if (hsl.z < 0.5)
            f2 = hsl.z * (1.0 + hsl.y);
        else
            f2 = (hsl.z + hsl.y) - (hsl.y * hsl.z);
        
        highp float f1 = 2.0 * hsl.z - f2;
        
        rgb.r = HueToRGB(f1, f2, hsl.x + (1.0/3.0));
        rgb.g = HueToRGB(f1, f2, hsl.x);
        rgb.b= HueToRGB(f1, f2, hsl.x - (1.0/3.0));
    }
    
    return rgb;
}



// Color Mode keeps the brightness of the base color and applies both the hue and saturation of the blend color.
highp vec3 BlendColor(highp vec3 base, highp vec3 blend)
{
    highp vec3 blendHSL = RGBToHSL(blend);
    return HSLToRGB(vec3(blendHSL.r, blendHSL.g, RGBToHSL(base).b));
}


void main()
{
	highp vec4 ble = texture2D( texBlend, texCoord2 );
	highp vec4 rgb =  texture2D(texInput, texCoord );
    gl_FragColor = vec4( mix( rgb.rgb, BlendColor(rgb.rgb, ble.rgb), blendPower * ble.a ), 1.0);
    //gl_FragColor = vec4( 1,0,1,1 );
}